/************************************************************* 
* ElectronicBattery.pde - simple electronic circuit          *
* class for junction                                         *
* written in Processing                                      *
* http://www.penguintutor.com/software.php                   *
* This is part of circuit1.pde software                      *
*************************************************************/
// Copyright 2010 Stewart Watkiss
/*  
    The circuit1.pde softweare is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License as published
    by the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This code is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this software.  If not, see <http://www.gnu.org/licenses/>.
*/


// point where 2 or more wires connect
// effectively a connection point / terminal etc.
class ElectronicJunction {
	// position of first connection (default left of switch)
	// initial setting -1 do not display
	int positionX = -1;
	int positionY = -1;
	
	// number of connections (if < 2 don't display circle - if >= 2 do)
	int connections = 0;
	
	public ElectronicJunction() {
	}

	// Constructor with initial position values
	public ElectronicJunction(int x, int y) {
		positionX = x;
		positionY = y;
	}
	
	void setPosition (int x, int y) {
		positionX = x;
		positionY = y;

	}
	
	int getPositionX () {
		return positionX;
	}

	int getPositionY () {
		return positionY;
	}
	
	void setConnections (int newConnections) {
		connections = newConnections;
	}
	
	void addConnections (int numConnections) {
		connections += numConnections;
	}

	
	int getConnections () {
		return connections;
	}

	void deleteConnections (int numConnections) {
		connections -= numConnections;
		// we cannot have less than 0 connections
		if (connections < 0) {connections = 0;}
	}
	
	void display() {
		// if positioned off screen then we don't display - object may have been created for future use
		if (positionX < 0 || positionY < 0 ) {return;}
		// only display if connections is greater than 2 (ie. a junction)
		if (connections > 2) {
			fill (#000000);
			ellipse (positionX, positionY, 5, 5);
		}
	}
}
