/************************************************************* 
* ElectronicLamp.pde - simple electronic circuit             *
* class for lampt                                            *
* written in Processing                                      *
* http://www.penguintutor.com/software.php                   *
* This is part of circuit1.pde software                      *
*************************************************************/
// Copyright 2010 Stewart Watkiss
/*  
    The circuit1.pde softweare is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License as published
    by the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This code is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this software.  If not, see <http://www.gnu.org/licenses/>.
*/

// this version of the class only works with circuit 1 (changes for future circuits)
class ElectronicLamp {
	// position of positive connection (default top of lamp)
	// initial setting -1 do not display
	int positionX = -1;
	int positionY = -1;
	int radius = 20;
	float[] point1, point2, point3, point4;
	
	// 0 = off, 1 =on
	int status = 0;
	
	public ElectronicLamp() {
		calcPoints ();
	}
	
	public ElectronicLamp (int x, int y) {
		positionX = x;
		positionY = y;
		calcPoints();
	}
	
	int getStartX () {
		return positionX;
	}
	
	int getStartY () {
		return positionY;
	}
	
	// end position is y + 2 * radius
	int getEndX () {
		return positionX;
	}
	
	int getEndY () {
		return positionY + (2 * radius);
	}
	
	// work out co-ordinates for cross
	// do at start so we dont' need to keep repeating
	void calcPoints () {
		point1 = pointCircle (radius, PI/ 4);
		point2 = pointCircle (radius, 3 * PI / 4);
		point3 = pointCircle (radius, 5 * PI / 4);
		point4 = pointCircle (radius, 7 * PI / 4);
	}
	
	// position is of left by default
	void setPosition (int x, int y) {
		positionX = x;
		positionY = y;

	}

	
	void setStatus (int newStatus) {
		status = newStatus;
	}

	
	int getStatus () {
		return status;
	}

	
	// If any on we turn off, if off we turn on 1
	void toggleStatus () {
		if (status == 0) {status = 1;}
		else {status = 0;}
	}


	void display () {
		// if positioned off screen then we don't display - object may have been created for future use
		if (positionX < 0 || positionY < 0 ) {return;}
		if (status == 0) {fill (colorBackground);}
		else {fill (#ffffff);}
		ellipseMode(CENTER); // This is default but we set now inc ase changed elsewhere
		
		ellipse (positionX, positionY + radius, (2*radius), (2*radius));
		line (positionX + point1[0], positionY + radius + point1[1], positionX + + point3[0], positionY + radius + point3[1]);
		line (positionX + point2[0], positionY + radius + point2[1], positionX + + point4[0], positionY + radius + point4[1]); 
	}


// Note radians 45deg is PI/4
float[] pointCircle(float radius, float radians) {
	float nx=radius*cos(radians);
	float ny=radius*sin(radians);
	float[] returnArray = {nx,ny};
	return returnArray;
   } 


}




