/************************************************************* 
* ElectronicVisualSwitch.pde - simple electronic circuit     *
* class for visual switch                                    *
* written in Processing                                      *
* http://www.penguintutor.com/software.php                   *
* This is part of circuit1.pde software                      *
*************************************************************/
// Copyright 2010 Stewart Watkiss
/*  
    The circuit1.pde softweare is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License as published
    by the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This code is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this software.  If not, see <http://www.gnu.org/licenses/>.
*/

class ElectronicVisualSwitch {
	// position of first connection (default left of switch)
	// initial setting -1 do not display
	int positionX = -1;
	int positionY = -1;
	
	int status = 0;	// 0 = off (open), 1 = on (closed)
	
	// Are we hovering over - need to track as we only change cursor back if we were hovering, but aren't any more
	int hover = 0;
	
	/*PImage onImage, offImage;*/
	PFont fontSwitchText;

	public ElectronicVisualSwitch() {
		/*onImage = loadImage("switch-on.gif");
		offImage = loadImage("switch-off.gif");*/
		fontSwitchText = loadFont("Dialog.bold-14.vlw");
	}

	// Constructor with initial position values
	public ElectronicVisualSwitch(int x, int y) {
		positionX = x;
		positionY = y;
		/*onImage = loadImage("switch-on.gif");
		offImage = loadImage("switch-off.gif");*/
		fontSwitchText = loadFont("Dialog.bold-14.vlw");
	}
	
	void setPosition (int x, int y) {
		positionX = x;
		positionY = y;

	}
	
	void setStatus (int newStatus) {
		status = newStatus;
	}
	
	int getStatus () {
		return status;
	}
	
	// If any on we turn off, if off we turn on 1
	void toggleStatus () {
		if (status == 0) {status = 1;}
		else {status = 0;}
	}
	
	void display() {
		// if positioned off screen then we don't display - object may have been created for future use
		if (positionX < 0 || positionY < 0 ) {return;}
		// draw outside of switch
		fill (#96c7e3);
		rect (positionX, positionY, 80, 30);
		// two rects 
		strokeWeight(2);
		if (status == 1) {fill (#c0d3de);}
		else {fill (#96c7e3);}
		rect (positionX+4, positionY + 4, 36, 24);
		if (status == 0) {fill (#c0d3de);}
		else {fill (#96c7e3);}
		rect (positionX+40, positionY + 4, 37, 24);
		strokeWeight(1);
		// Add text to switch
		textFont (fontSwitchText);
		fill (#000000);
		text ("On", positionX + 9, positionY + 23);
		text ("Off", positionX + 47, positionY + 23);
		/*if (status == 0) {
			image(offImage, positionX, positionY, 81, 33);
		} else {
			image(onImage, positionX, positionY, 81, 33);
		}*/

	}
	
	
	// return 0 if not hovering, 1 if over on and 2 if over off
	// also change cursor 
	// note this is not same numbering as status as we use 0 for not hovering
	int mouseOver (int mouseX, int mouseY) {
		// Are we hovering over on button
		if ((mouseX > positionX) && (mouseX < positionX + 40) && (mouseY > positionY) && (mouseY <positionY + 30)) {
			// is on button not selected (we only hover on unselected side of button
			if (status == 0) {
				hover = 1;
				// change cursor
				cursor(HAND);
				return 1;
			}
			
		}
		// Are we hovering over off button
		if (mouseX > positionX + 40 && mouseX < positionX + 80 && mouseY > positionY && mouseY <positionY + 30) {
			// is off button not selected (we only hover on unselected side of button
			if (status == 1) {
				hover = 1;
				// change cursor
				cursor(HAND);
				return 2;
			}
			
		}
	
		// if we reach here then we are not hovering
		// if we were hovering change the cursor back
		if (hover == 1) {
			hover = 0;
			cursor(ARROW);
		}
		return 0;
	}
	
	
	// return 0 if not on button return 1 if on button pressed, return 2 if off button pressed
	// note return is not same numbering as status as we use 0 for not hovering
	// we update status in class
	// note we don't wait until mouseReleased - this is simulating a real switch, not a computer button
	int mousePressed (int mouseX, int mouseY) {
		// Are we hovering?
		int newStatus = this.mouseOver (mouseX, mouseY);
		// only change if button was pressed (not 0)
		if (newStatus != 0) {
			// convert the status from button status to selected status
			if (newStatus == 1) {status = 1;}
			if (newStatus == 2) {status = 0;}
		}
		return (newStatus);
	}
	
	
} // end class
