/************************************************************* 
* circuit1.pde - simple electronic circuit                   * 
* written in Processing                                      *
* http://www.penguintutor.com/software.php                   *
* This is part of circuit1.pde software                      *
*************************************************************/
// Copyright 2010 Stewart Watkiss
/*  
    The circuit1.pde softweare is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License as published
    by the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This code is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this software.  If not, see <http://www.gnu.org/licenses/>.
*/



import processing.core.*; 
import processing.xml.*; 

import java.applet.*; 
import java.awt.*; 
import java.awt.image.*; 
import java.awt.event.*; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class circuit1 extends PApplet {

PFont fontTitle, fontTooltip;
PFont fontPT;
int colorBackground = 0xffA9A9F5;
int colorTextTitle = color (0xff000000);
int colorTextTooltip = color (0xff000000);
int[] circuitColor = {colorBackground, 0xff999999, 0xff333333, 0xff000000};
String titleText = "Basic electronic circuit";
String PTText = "www.penguintutor.com";
int screenWidth = 600;
int screenHeight = 400;
int diagStartX = (screenWidth -300) / 2;
int diagStartY = 130;


// number of connection points (eg. turn corners or junctions) and joining links
// used to determine array size for junctions
// this may need to be increased if using this code as the basis for a larger circuit
// or we could use expand to add new objects ?
int maxJunctions = 10;
int maxWires = 10;

// so we can keep count and display the correct number
int numJunctions = 0;
int numWires = 0;

ElectronicSwitch switch1;
ElectronicBattery battery1;
ElectronicLamp lamp1;
ElectronicJunction[] junctions;
ElectronicWire[] wires;
ElectronicVisualSwitch visualSwitch1;

public void setup() {
	frameRate(10);

	
	fontTitle = loadFont("Dialog.bold-24.vlw");
	fontTooltip = loadFont("Dialog.plain-10.vlw");
	fontPT = loadFont("Dialog.plain-12.vlw");
	size (screenWidth, screenHeight);

	// note that this must be in setup - not before as font directory not loaded
	battery1 = new ElectronicBattery (diagStartX, diagStartY+80);
	switch1 = new ElectronicSwitch (diagStartX+100, diagStartY);
	lamp1 = new ElectronicLamp (diagStartX+250, diagStartY+80);
	junctions = new ElectronicJunction[maxJunctions];
	wires = new ElectronicWire[maxWires];
	visualSwitch1 = new ElectronicVisualSwitch(diagStartX+75, diagStartY-60);
	
	// These are all defined manually
	// We may put these into a file or specific format to make it easier	
	junctions[0] = new ElectronicJunction(diagStartX, diagStartY);
	numJunctions ++;
	junctions[1] = new ElectronicJunction(diagStartX + 250, diagStartY);
	numJunctions ++;
	junctions[2] = new ElectronicJunction(diagStartX + 250, diagStartY + 200);
	numJunctions ++;
	junctions[3] = new ElectronicJunction(diagStartX, diagStartY + 200);
	numJunctions ++;
	
	//Battery to top left
	wires[0] = new ElectronicWire(battery1.getStartX(), battery1.getStartY(), junctions[0].getPositionX(), junctions[0].getPositionY());
	numWires ++;
	// top left to switch
	wires[1] = new ElectronicWire(junctions[0].getPositionX(),junctions[0].getPositionY(),switch1.getStartX(), switch1.getStartY());
	numWires ++;
	// switch to top right
	wires[2] = new ElectronicWire(switch1.getEndX(), switch1.getEndY(),junctions[1].getPositionX(),junctions[1].getPositionY());
	numWires ++;
	// top right to light
	wires[3] = new ElectronicWire(junctions[1].getPositionX(),junctions[1].getPositionY(), lamp1.getStartX(),lamp1.getStartY());
	numWires ++;
	junctions[1].addConnections (2);
	// light to bottom right
	wires[4] = new ElectronicWire(lamp1.getEndX(),lamp1.getEndY(),junctions[2].getPositionX(),junctions[2].getPositionY());
	numWires ++;
	// bottom right to bottom left
	wires[5] = new ElectronicWire(junctions[2].getPositionX(),junctions[2].getPositionY(),junctions[3].getPositionX(),junctions[3].getPositionY());
	numWires ++;
	junctions[2].addConnections (2);
	// bottom left to battery
	wires[6] = new ElectronicWire(junctions[3].getPositionX(),junctions[3].getPositionY(),battery1.getEndX(),battery1.getEndY());
	numWires ++;
	junctions[3].addConnections (2);
}

public void draw() {
	
	updateMouse();
	background (colorBackground);
	
	// check status of visual switch
	switch1.setStatus(visualSwitch1.getStatus());
	
	// Title
	textFont(fontTitle);	
	fill (colorTextTitle);
	float titleWidth = textWidth (titleText);
	text (titleText, (screenWidth - titleWidth) / 2, 30);
	textFont(fontPT);
	text (PTText, 30, screenHeight - 20);
	
	switch1.display();
	battery1.display();
	lamp1.display();
	// loop over all junctions display
	for (int i=0; i < numJunctions; i++) {
		junctions[i].display();
	}
	for (int i=0; i < numWires; i++) {
		wires[i].display();
	}
	visualSwitch1.display();
	//switch1.toggleStatus();
	//visualSwitch1.toggleStatus();
	lamp1.setStatus (switch1.getStatus());
}


// make any mouse status changes - called from draw() each cycle                                                                  
public void updateMouse(){
	visualSwitch1.mouseOver(mouseX, mouseY);
}


public void mousePressed()
{
	visualSwitch1.mousePressed(mouseX, mouseY);
}

public void mouseReleased()
{
	//visualSwitch simulates a real switch not a computer switch - so doesn't matter if released
 
}







// this version of the class only works with circuit 1 (changes for future circuits)
class ElectronicBattery {
	// position of positive connection (default top of battery)
	// initial setting -1 do not display
	int positionX = -1;
	int positionY = -1;
	
	PFont fontSymbolText;
	
	public ElectronicBattery() {
		fontSymbolText = loadFont("Dialog.plain-10.vlw");
	}
	
	public ElectronicBattery (int x, int y) {
		positionX = x;
		positionY = y;
		fontSymbolText = loadFont("Dialog.plain-10.vlw");
	}
	
	
	public void setPosition (int x, int y) {
		positionX = x;
		positionY = y;
	}
	
	public int getStartX () {
		return positionX;
	}
	
	public int getStartY () {
		return positionY;
	}
	
	// end position is y + 35
	public int getEndX () {
		return positionX;
	}
	
	public int getEndY () {
		return positionY + 35;
	}
	
	
	public void display () {
		// if positioned off screen then we don't display - object may have been created for future use
		if (positionX < 0 || positionY < 0 ) {return;}
		line (positionX - 20, positionY, positionX + 20, positionY);
		line (positionX - 10, positionY + 5, positionX + 10, positionY + 5);
		line (positionX, positionY + 10, positionX, positionY + 15);
		line (positionX, positionY + 20, positionX, positionY + 25);
		line (positionX - 20, positionY + 30, positionX + 20, positionY + 30);
		line (positionX - 10, positionY + 35, positionX + 10, positionY + 35);
		// add + sign
		fill (0xff000000);
		textFont (fontSymbolText);
		text ("+", positionX + 10, positionY -3 );
				
	}
		
	
	
}
// point where 2 or more wires connect
// effectively a connection point / terminal etc.
class ElectronicJunction {
	// position of first connection (default left of switch)
	// initial setting -1 do not display
	int positionX = -1;
	int positionY = -1;
	
	// number of connections (if < 2 don't display circle - if >= 2 do)
	int connections = 0;
	
	public ElectronicJunction() {
	}

	// Constructor with initial position values
	public ElectronicJunction(int x, int y) {
		positionX = x;
		positionY = y;
	}
	
	public void setPosition (int x, int y) {
		positionX = x;
		positionY = y;

	}
	
	public int getPositionX () {
		return positionX;
	}

	public int getPositionY () {
		return positionY;
	}
	
	public void setConnections (int newConnections) {
		connections = newConnections;
	}
	
	public void addConnections (int numConnections) {
		connections += numConnections;
	}

	
	public int getConnections () {
		return connections;
	}

	public void deleteConnections (int numConnections) {
		connections -= numConnections;
		// we cannot have less than 0 connections
		if (connections < 0) {connections = 0;}
	}
	
	public void display() {
		// if positioned off screen then we don't display - object may have been created for future use
		if (positionX < 0 || positionY < 0 ) {return;}
		// only display if connections is greater than 2 (ie. a junction)
		if (connections > 2) {
			fill (0xff000000);
			ellipse (positionX, positionY, 5, 5);
		}
	}
}
// this version of the class only works with circuit 1 (changes for future circuits)
class ElectronicLamp {
	// position of positive connection (default top of lamp)
	// initial setting -1 do not display
	int positionX = -1;
	int positionY = -1;
	int radius = 20;
	float[] point1, point2, point3, point4;
	
	// 0 = off, 1 =on
	int status = 0;
	
	public ElectronicLamp() {
		calcPoints ();
	}
	
	public ElectronicLamp (int x, int y) {
		positionX = x;
		positionY = y;
		calcPoints();
	}
	
	public int getStartX () {
		return positionX;
	}
	
	public int getStartY () {
		return positionY;
	}
	
	// end position is y + 2 * radius
	public int getEndX () {
		return positionX;
	}
	
	public int getEndY () {
		return positionY + (2 * radius);
	}
	
	// work out co-ordinates for cross
	// do at start so we dont' need to keep repeating
	public void calcPoints () {
		point1 = pointCircle (radius, PI/ 4);
		point2 = pointCircle (radius, 3 * PI / 4);
		point3 = pointCircle (radius, 5 * PI / 4);
		point4 = pointCircle (radius, 7 * PI / 4);
	}
	
	// position is of left by default
	public void setPosition (int x, int y) {
		positionX = x;
		positionY = y;

	}

	
	public void setStatus (int newStatus) {
		status = newStatus;
	}

	
	public int getStatus () {
		return status;
	}

	
	// If any on we turn off, if off we turn on 1
	public void toggleStatus () {
		if (status == 0) {status = 1;}
		else {status = 0;}
	}


	public void display () {
		// if positioned off screen then we don't display - object may have been created for future use
		if (positionX < 0 || positionY < 0 ) {return;}
		if (status == 0) {fill (colorBackground);}
		else {fill (0xffffffff);}
		ellipseMode(CENTER); // This is default but we set now inc ase changed elsewhere
		
		ellipse (positionX, positionY + radius, (2*radius), (2*radius));
		line (positionX + point1[0], positionY + radius + point1[1], positionX + + point3[0], positionY + radius + point3[1]);
		line (positionX + point2[0], positionY + radius + point2[1], positionX + + point4[0], positionY + radius + point4[1]); 
	}


// Note radians 45deg is PI/4
public float[] pointCircle(float radius, float radians) {
	float nx=radius*cos(radians);
	float ny=radius*sin(radians);
	float[] returnArray = {nx,ny};
	return returnArray;
   } 


}




class ElectronicSwitch {
	// position of first connection (default left of switch)
	// initial setting -1 do not display
	int positionX = -1;
	int positionY = -1;
	
	// Use int as it means we can add multi-way in future 
	// May need to do differently for mutlipole 
	int status = 1;	// 0 = open, 1 = closed
	
	public ElectronicSwitch() {
	}

	// Constructor with initial position values
	public ElectronicSwitch(int x, int y) {
		positionX = x;
		positionY = y;
	}
	
	public void setPosition (int x, int y) {
		positionX = x;
		positionY = y;

	}
	
	
	public int getStartX () {
		return positionX;
	}
	
	public int getStartY () {
		return positionY;
	}
	
	// end position is x + 40
	public int getEndX () {
		return positionX + 40;
	}
	
	public int getEndY () {
		return positionY;
	}
	
	
	public void setStatus (int newStatus) {
		status = newStatus;
	}
	
	public int getStatus () {
		return status;
	}
	
	// If any on we turn off, if off we turn on 1
	public void toggleStatus () {
		if (status == 0) {status = 1;}
		else {status = 0;}
	}
	
	public void display() {
		// if positioned off screen then we don't display - object may have been created for future use
		if (positionX < 0 || positionY < 0 ) {return;}
		fill (colorBackground);
		ellipse (positionX, positionY, 5, 5);
		ellipse (positionX + 40, positionY, 5, 5); 
		if (status == 1) { line (positionX, positionY-2, positionX + 40, positionY - 2); } 
		else {line (positionX, positionY-2, positionX + 40, positionY - 20);}
	}
}
class ElectronicVisualSwitch {
	// position of first connection (default left of switch)
	// initial setting -1 do not display
	int positionX = -1;
	int positionY = -1;
	
	int status = 0;	// 0 = off (open), 1 = on (closed)
	
	// Are we hovering over - need to track as we only change cursor back if we were hovering, but aren't any more
	int hover = 0;
	
	/*PImage onImage, offImage;*/
	PFont fontSwitchText;

	public ElectronicVisualSwitch() {
		/*onImage = loadImage("switch-on.gif");
		offImage = loadImage("switch-off.gif");*/
		fontSwitchText = loadFont("Dialog.bold-14.vlw");
	}

	// Constructor with initial position values
	public ElectronicVisualSwitch(int x, int y) {
		positionX = x;
		positionY = y;
		/*onImage = loadImage("switch-on.gif");
		offImage = loadImage("switch-off.gif");*/
		fontSwitchText = loadFont("Dialog.bold-14.vlw");
	}
	
	public void setPosition (int x, int y) {
		positionX = x;
		positionY = y;

	}
	
	public void setStatus (int newStatus) {
		status = newStatus;
	}
	
	public int getStatus () {
		return status;
	}
	
	// If any on we turn off, if off we turn on 1
	public void toggleStatus () {
		if (status == 0) {status = 1;}
		else {status = 0;}
	}
	
	public void display() {
		// if positioned off screen then we don't display - object may have been created for future use
		if (positionX < 0 || positionY < 0 ) {return;}
		// draw outside of switch
		fill (0xff96c7e3);
		rect (positionX, positionY, 80, 30);
		// two rects 
		strokeWeight(2);
		if (status == 1) {fill (0xffc0d3de);}
		else {fill (0xff96c7e3);}
		rect (positionX+4, positionY + 4, 36, 24);
		if (status == 0) {fill (0xffc0d3de);}
		else {fill (0xff96c7e3);}
		rect (positionX+40, positionY + 4, 37, 24);
		strokeWeight(1);
		// Add text to switch
		textFont (fontSwitchText);
		fill (0xff000000);
		text ("On", positionX + 9, positionY + 23);
		text ("Off", positionX + 47, positionY + 23);
		/*if (status == 0) {
			image(offImage, positionX, positionY, 81, 33);
		} else {
			image(onImage, positionX, positionY, 81, 33);
		}*/

	}
	
	
	// return 0 if not hovering, 1 if over on and 2 if over off
	// also change cursor 
	// note this is not same numbering as status as we use 0 for not hovering
	public int mouseOver (int mouseX, int mouseY) {
		// Are we hovering over on button
		if ((mouseX > positionX) && (mouseX < positionX + 40) && (mouseY > positionY) && (mouseY <positionY + 30)) {
			// is on button not selected (we only hover on unselected side of button
			if (status == 0) {
				hover = 1;
				// change cursor
				cursor(HAND);
				return 1;
			}
			
		}
		// Are we hovering over off button
		if (mouseX > positionX + 40 && mouseX < positionX + 80 && mouseY > positionY && mouseY <positionY + 30) {
			// is off button not selected (we only hover on unselected side of button
			if (status == 1) {
				hover = 1;
				// change cursor
				cursor(HAND);
				return 2;
			}
			
		}
	
		// if we reach here then we are not hovering
		// if we were hovering change the cursor back
		if (hover == 1) {
			hover = 0;
			cursor(ARROW);
		}
		return 0;
	}
	
	
	// return 0 if not on button return 1 if on button pressed, return 2 if off button pressed
	// note return is not same numbering as status as we use 0 for not hovering
	// we update status in class
	// note we don't wait until mouseReleased - this is simulating a real switch, not a computer button
	public int mousePressed (int mouseX, int mouseY) {
		// Are we hovering?
		int newStatus = this.mouseOver (mouseX, mouseY);
		// only change if button was pressed (not 0)
		if (newStatus != 0) {
			// convert the status from button status to selected status
			if (newStatus == 1) {status = 1;}
			if (newStatus == 2) {status = 0;}
		}
		return (newStatus);
	}
	
	
} // end class
// point where 2 or more wires connect
// effectively a connection point / terminal etc.
class ElectronicWire {
	// position of first connection 
	int startX = -1;
	int startY = -1;
	// position of last connection
	int endX = -1;
	int endY = -1;
	
	// positionX is where a label will go if selected
	// by leaving at -1 we don't show any label
	int positionX = -1;
	int positionY = -1;
	
	// number of connections (if < 2 don't display circle - if >= 2 do)
	int connections = 0;
	
	public ElectronicWire() {
	}

	// Constructor with initial position values 
	// note this does not set position which is of text label
	// insetad we set start and end points - if label required we set seperately
	public ElectronicWire(int newstartX, int newstartY, int newendX, int newendY) {
		startX = newstartX;
		startY = newstartY;
		endX = newendX;
		endY = newendY;
	}
	
	// sets position of label
	public void setPosition (int x, int y) {
		positionX = x;
		positionY = y;

	}
	
	public void setStart (int newstartX, int newstartY) {
		startX = newstartX;
		startY = newstartY;
	}
	
	public void setEnd (int newendX, int newendY) {
		endX = newendX;
		endY = newendY;
	}
	
	
	public void display() {
		// Position will be used for label in future
		//if (positionX < 0 || positionY < 0 ) {return;}
		// draw line from start position to end position
		line (startX, startY, endX, endY);
	}
}

  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#DFDFDF", "circuit1" });
  }
}
