/************************************************************* 
* circuit1.pde - simple electronic circuit                   * 
* written in Processing                                      *
* http://www.penguintutor.com/software.php                   *
* This is part of circuit1.pde software                      *
*************************************************************/
// Copyright 2010 Stewart Watkiss
/*  
    The circuit1.pde softweare is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License as published
    by the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This code is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this software.  If not, see <http://www.gnu.org/licenses/>.
*/


PFont fontTitle, fontTooltip;
PFont fontPT;
int colorBackground = #A9A9F5;
color colorTextTitle = color (#000000);
color colorTextTooltip = color (#000000);
int[] circuitColor = {colorBackground, #999999, #333333, #000000};
String titleText = "Basic electronic circuit";
String PTText = "www.penguintutor.com";
int screenWidth = 600;
int screenHeight = 400;
int diagStartX = (screenWidth -300) / 2;
int diagStartY = 130;


// number of connection points (eg. turn corners or junctions) and joining links
// used to determine array size for junctions
// this may need to be increased if using this code as the basis for a larger circuit
// or we could use expand to add new objects ?
int maxJunctions = 10;
int maxWires = 10;

// so we can keep count and display the correct number
int numJunctions = 0;
int numWires = 0;

ElectronicSwitch switch1;
ElectronicBattery battery1;
ElectronicLamp lamp1;
ElectronicJunction[] junctions;
ElectronicWire[] wires;
ElectronicVisualSwitch visualSwitch1;

void setup() {
	frameRate(10);

	
	fontTitle = loadFont("Dialog.bold-24.vlw");
	fontTooltip = loadFont("Dialog.plain-10.vlw");
	fontPT = loadFont("Dialog.plain-12.vlw");
	size (screenWidth, screenHeight);

	// note that this must be in setup - not before as font directory not loaded
	battery1 = new ElectronicBattery (diagStartX, diagStartY+80);
	switch1 = new ElectronicSwitch (diagStartX+100, diagStartY);
	lamp1 = new ElectronicLamp (diagStartX+250, diagStartY+80);
	junctions = new ElectronicJunction[maxJunctions];
	wires = new ElectronicWire[maxWires];
	visualSwitch1 = new ElectronicVisualSwitch(diagStartX+75, diagStartY-60);
	
	// These are all defined manually
	// We may put these into a file or specific format to make it easier	
	junctions[0] = new ElectronicJunction(diagStartX, diagStartY);
	numJunctions ++;
	junctions[1] = new ElectronicJunction(diagStartX + 250, diagStartY);
	numJunctions ++;
	junctions[2] = new ElectronicJunction(diagStartX + 250, diagStartY + 200);
	numJunctions ++;
	junctions[3] = new ElectronicJunction(diagStartX, diagStartY + 200);
	numJunctions ++;
	
	//Battery to top left
	wires[0] = new ElectronicWire(battery1.getStartX(), battery1.getStartY(), junctions[0].getPositionX(), junctions[0].getPositionY());
	numWires ++;
	// top left to switch
	wires[1] = new ElectronicWire(junctions[0].getPositionX(),junctions[0].getPositionY(),switch1.getStartX(), switch1.getStartY());
	numWires ++;
	// switch to top right
	wires[2] = new ElectronicWire(switch1.getEndX(), switch1.getEndY(),junctions[1].getPositionX(),junctions[1].getPositionY());
	numWires ++;
	// top right to light
	wires[3] = new ElectronicWire(junctions[1].getPositionX(),junctions[1].getPositionY(), lamp1.getStartX(),lamp1.getStartY());
	numWires ++;
	junctions[1].addConnections (2);
	// light to bottom right
	wires[4] = new ElectronicWire(lamp1.getEndX(),lamp1.getEndY(),junctions[2].getPositionX(),junctions[2].getPositionY());
	numWires ++;
	// bottom right to bottom left
	wires[5] = new ElectronicWire(junctions[2].getPositionX(),junctions[2].getPositionY(),junctions[3].getPositionX(),junctions[3].getPositionY());
	numWires ++;
	junctions[2].addConnections (2);
	// bottom left to battery
	wires[6] = new ElectronicWire(junctions[3].getPositionX(),junctions[3].getPositionY(),battery1.getEndX(),battery1.getEndY());
	numWires ++;
	junctions[3].addConnections (2);
}

void draw() {
	
	updateMouse();
	background (colorBackground);
	
	// check status of visual switch
	switch1.setStatus(visualSwitch1.getStatus());
	
	// Title
	textFont(fontTitle);	
	fill (colorTextTitle);
	float titleWidth = textWidth (titleText);
	text (titleText, (screenWidth - titleWidth) / 2, 30);
	textFont(fontPT);
	text (PTText, 30, screenHeight - 20);
	
	switch1.display();
	battery1.display();
	lamp1.display();
	// loop over all junctions display
	for (int i=0; i < numJunctions; i++) {
		junctions[i].display();
	}
	for (int i=0; i < numWires; i++) {
		wires[i].display();
	}
	visualSwitch1.display();
	//switch1.toggleStatus();
	//visualSwitch1.toggleStatus();
	lamp1.setStatus (switch1.getStatus());
}


// make any mouse status changes - called from draw() each cycle                                                                  
void updateMouse(){
	visualSwitch1.mouseOver(mouseX, mouseY);
}


void mousePressed()
{
	visualSwitch1.mousePressed(mouseX, mouseY);
}

void mouseReleased()
{
	//visualSwitch simulates a real switch not a computer switch - so doesn't matter if released
 
}







